/*
 * attrextr (attribute extract)
 * written by Ramshankar <ramshankar@bemail.org>
 *
*/

#include <fstream.h>
#include <Entry.h>
#include <Node.h>
#include <Directory.h>
#include <File.h>
#include <Path.h>
#include <Mime.h>
#include <String.h>
#include <stdio.h>

/* Protos */
BString SizeString (off_t size);

int main (int argc, char **argv)
{
	/* Check args */
	if (argc < 3)
	{
		printf ("%s v0.1 - by Ramshankar (Somebody)\n", argv[0]+2);
		printf ("usage: %s <indir> <outfile>\n", argv[0]);
		printf ("\textracts some standard attributes from all entries in folder <indir>\n");
		printf ("\tand dumps them to file <outfile>\n");
		return -1;
	}
	
	
	/* Things we need */
	BEntry indir, entry;
	BDirectory infolder;
	BPath path, parentpath;
	BNode node;
	BMimeType mt;
	off_t filesize;
	char description [B_MIME_TYPE_LENGTH];
	void *buffer;
	fstream f;


	/* Some initializations */
	indir.SetTo(argv[1], true);
	

	/* Validity checks */
	if (indir.IsDirectory() == false)
	{
		printf ("%s is not a valid directory path", argv[1]);
		return -1;
	}
	

	/* Initialize necessary stuff some more checks */
	f.open (argv[2], ios::out);
	if (f == NULL)
	{
		printf ("error: could not initialize %s\n", argv[2]);
		return -1;
	}

	infolder.SetTo (&indir);
	if (infolder.CountEntries() == 0)
	{
		printf ("%s has no entries\n", argv[1]);
		return -1;
	}
	
	
	/* Finally!, loop through and extract attribs and write */
	while (infolder.GetNextEntry (&entry, true) == B_OK)
	{
		entry.GetPath(&path);
		node.SetTo (path.Path());
		node.ReadAttr ("BEOS:TYPE", B_STRING_TYPE, 0, &buffer, 255);
		f << "name: " << path.Leaf() << endl;

		mt.SetTo ((char*)&buffer);
		mt.GetShortDescription (description);
		f << "type: " << description << endl;
		
		entry.GetSize (&filesize);
		f << "size: " << SizeString(filesize).String() << endl;
		
		path.GetParent(&parentpath);
		f << "path: " << parentpath.Path() << endl << endl;
	}

	f.close();
}

/*========================================================================================*/

BString SizeString (off_t v)
{
	/* Hacked & tweaked from BeShare -- thanks Jeremy! */
	char buf[256];
	if (v > (1024LL * 1024LL * 1024LL * 1024LL))
		sprintf (buf, "%.2f TB", ((double)v) / (1024LL * 1024LL * 1024LL * 1024LL));
	else if (v > (1024LL * 1024LL * 1024LL))
		sprintf(buf, "%.2f GB", ((double)v) / (1024LL * 1024LL * 1024LL));
	else if (v > (1024LL * 1024LL))
		sprintf(buf, "%.2f MB", ((double)v) / (1024LL * 1024LL));
	else if (v > (1024LL))
		sprintf(buf, "%.2f KB", ((double)v) / 1024LL);
	else
		sprintf(buf, "%Li bytes", v);
	
	BString str;
	str << buf;
	
	return str;
}

/*========================================================================================*/
